#!/usr/bin/env python3

' Fetch app switches config from server.'

import fcntl
import json
import os
import requests
import sys
import syslog

class pLock(object):
    def __init__(self, lockfile):
        self.lockfile = lockfile
        self.fd = os.open(lockfile, os.O_RDWR | os.O_CREAT)
 
    def lock(self):
        try:
            fcntl.lockf(self.fd, fcntl.LOCK_EX | fcntl.LOCK_NB)
            return True
        except:
            self.fd.close()
            return False

    def unlock(self):
        fcntl.lockf(self.fd, fcntl.LOCK_UN)

    def __del__(self):
        os.close(self.fd)
        os.unlink(self.lockfile)

################################################################################
def parseVersion(seq):
    for i, item in enumerate(seq):
        if ((item.get('app') == 'switch-config')):
            return int(item.get('version'))

def getLocalVersion():
    if not os.access('/etc/app-switches.json', os.F_OK):
        return -1
    with open('/etc/app-switches.json', 'r') as f:        
        cfg_obj = json.load(f)
    return parseVersion(cfg_obj['config'])
    
def updateSwitchConfig(config):
    with open('/etc/app-switches.json', 'w') as f:
        config_obj = json.loads(config)
        json.dump(config_obj, f, indent=4)

def main():
    lock = pLock('/var/lock/switch-fetch.lock');
    if (not lock.lock()):
        syslog.syslog('switch-fetch acquire lock failed!');
        sys.exit(-1)
    try:
        response = requests.get('http://124.16.141.197:3000/switch')
        remote_cfg = json.loads(response.text)
        version = parseVersion(remote_cfg['config'])
        local_version = getLocalVersion()   
        if (version > local_version):
            updateSwitchConfig(response.text)
    finally:
        lock.unlock()

################################################################################
if __name__=='__main__':
    main()
